/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/

package casa.dodwan.napi.tcp;

import casa.dodwan.napi.PduSerializer;
import casa.dodwan.napi.DodwanNapiEndpoint;
import casa.dodwan.napi.CommandExecutor;
import casa.dodwan.napi.CommandExecutor.CommandException;
import casa.dodwan.napi.Pdu;
import casa.dodwan.run.Dodwan;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;


/**
 * Handler that implements the conversation between a connected client and the DoDWAN TCP Network API server
 */

public class DodwanTcpEndpoint extends Thread implements DodwanNapiEndpoint {
    
    public Dodwan dodwan ;
    private String client;
    private CommandExecutor commandExecutor;
    private InputStream in;
    private OutputStream out;
    private PduSerializer serializer;
    
    
    public DodwanTcpEndpoint(Socket socket) {
   
        try {
            this.out = socket.getOutputStream();
            this.in = socket.getInputStream();
        } catch (IOException ex) {
            System.err.println("DodwanTcpEndpoint: cannot get input or output stream");
            ex.printStackTrace(System.err);
        }
	
    
    }
    
   
    @Override
    public void run()  {
        
        this.dodwan = DodwanTcpServer.dodwan;
        this.serializer = DodwanTcpServer.pduSerializer;
        
        this.commandExecutor = new CommandExecutor(this);
        this.commandExecutor.init();
        
        boolean goahead = true;
        
        while (goahead) {
            try {
              this.commandExecutor.receiveAndExecuteCommand();
            }
            catch (CommandException ce) {
               System.err.println("DodwanTcpEndpoint command error when receiving and executing command: "+ce.getMessage()); 
            } catch (IOException ioe) {
                System.err.println("DodwanTcpEndpoint fatal error when receiving command: "+ioe.getMessage());
                goahead = false;
            }
            
        }
        
        
        
    }

   
      
    /*
    * Receive a PDU from the client
    */
    @Override
    public Pdu receivePdu() throws IOException {
        Pdu pdu = new Pdu();
        if (serializer.read(pdu, this.in) == -1) {
            this.in.close();
            throw new IOException();
        }
        return pdu;
    }
    
      
    /**
     *  Sends a PDU to the client
    */
    @Override
    public void sendPdu(Pdu pdu) {
            try {      
                serializer.write(pdu, this.out);
            } catch (IOException e) {
                System.err.println("Error when sending the " + pdu.getString("name") + " PDU on the socket");
                try {
                    this.out.close();
                } catch (IOException ex) {
                    System.err.println("Tried to close PDU outputstream");
                    e.printStackTrace(System.err);
                }
                e.printStackTrace(System.err);
            } 
    }
 
  
    @Override
    public Dodwan getDodwan() {
        return this.dodwan;
    }

    
}

