

/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/

package casa.dodwan.napi.tcp;


import casa.dodwan.napi.BsonPduSerializer;
import casa.dodwan.napi.DodwanNapiServer;
import casa.dodwan.napi.JsonPduSerializer;
import casa.dodwan.napi.PduSerializer;
import casa.dodwan.run.Dodwan;
import casa.dodwan.run.dodwand;
import casa.dodwan.util.ServiceEnvironment;
import casa.util.Logger;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;


/**
 * Dodwan plugin providing a TCP server for the DoDWAN Network API
 *
 */

public class DodwanTcpServer extends Thread  implements DodwanNapiServer {
   
    
   static public Dodwan dodwan = null;
   static public PduSerializer pduSerializer = null;
    
   private String host = null;
    
    // The port on which the server should listen.
    // 0 if the port is to be chosen by the system. In this case the chosen 
    // port is obtained by calling the getPort() method.
    private int port = 0;
    
  
    
    private ServerSocket serverSocket;
    
    private boolean alive;
    
    //-----------------------------------------------------------------------------
     
    public DodwanTcpServer(dodwand dodwand) {
       this.dodwan = dodwand.dodwan;
       
       ServiceEnvironment dodwanTcpServerEnvironment = new ServiceEnvironment("dodwan_napi_tcp");
       //DodwanEnvironment env =  DodwanEnvironment.getInstance();
       //ConfigurationProperties props = env.getProperties();
       this.port = dodwanTcpServerEnvironment.properties.getInteger("dodwan_napi_tcp.port", 8030);
     
       String serializingMethod = dodwanTcpServerEnvironment.properties.getString("dodwan_napi_tcp.serial_method", "json");
       switch (serializingMethod) {
           case "json":
               this.pduSerializer = new JsonPduSerializer();
               break;
            case "bson":
               this.pduSerializer = new BsonPduSerializer();
               break;  
            default:
                System.err.println("DodwanTcpServer: unknown serializing method "+ serializingMethod
                                   + ". Falling back to bson");
                this.pduSerializer = new BsonPduSerializer();
       }
              
           
       try {
           this.serverSocket = new ServerSocket(this.port);
       } catch (IOException ex) {
           System.err.println("DodwanTcpServer: cannot create ServerSocket");
           ex.printStackTrace(System.err);
       }
       
       if (this.port == 0) {
         this.port = this.serverSocket.getLocalPort();
         dodwanTcpServerEnvironment.properties.setInteger("dodwan_napi_tcp.port", this.port);
      }
       
      
      System.out.println("DodwanTcpServer listening on port "+this.port);
      dodwand.writePortFile("dodwan_napi_tcp.port", this.port);
      
    }

    
    
    
    public void startServer() {
        Logger.log("napi-tcp","Starting NAPI TCP server. Listening on port "+this.port);
        this.alive = true;
        start();
    }
    
   public void stopServer() {
        Logger.log("napi-tcp","Stopping NAPI TCP server.");
        this.alive = false;
       try {
           this.serverSocket.close();
       } catch (IOException ex) {
           System.err.println("Error when closing DodwanTcpServer");
           ex.printStackTrace(System.err);
       }
    }
    
    
    //-----------------------------------------------------------------------------
    /**
     * Gives the number of the real port the server listens to. 
     * It is the asked port or the port chosen by the system.
     * @return the port number 
     */
    public int getPort() {
        return this.port;
    }
    
    
    
    @Override
    public void run() {
                      
        Socket clientSocket = null;
	System.err.println("Starting Dodwan NAPI Tcp Server ");
	while (this.alive) {
            
	    try {
		clientSocket = serverSocket.accept();
		System.err.println("Dodwan NAPI Tcp Server got a new client");		
		DodwanTcpEndpoint  endpoint =  new DodwanTcpEndpoint(clientSocket);
		endpoint.start();
	    } catch (IOException e) {
		System.err.println("DodwanTcpServer.run(): " + e);
	    }
	}
    }
        
        
  
}
